/***************************************************************************
	usbua100mix.h  -  A Linux Mixer/Control Panel for the Roland UA-100
                             -------------------
    begin                : June 6, 2001
    copyright            : (C) 2001 by Michael Minn
    email                : michael@michaelminn.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

***************************************************************************/

#include "usbua100mix.h"
#include "usbua100mix.xpm"


/****************************** Channel Widgets **************************************/

usbua100mix_channel::usbua100mix_channel(usbua100mix *mixer, int channel, char *name) : QWidget(mixer, name)
{
	move(channel * 60, 25);
	resize(60, 250);
	setBackgroundMode(QWidget::PaletteMid);
	this->channel = channel;

	if (channel == 0)
    {
		input_mode = new QComboBox(this);
		input_mode->move(2, 2);
		input_mode->resize(56, 23);
		input_mode->setCurrentItem(0);
		input_mode->setEditable(FALSE);
		input_mode->insertItem("Mic1");
		input_mode->insertItem("Line");
		input_mode->insertItem("M1+2");
		connect(input_mode, SIGNAL(activated(int)), this, SLOT(slotModeChange(int)));
	}
	else
	{
		name_box = new QLineEdit(this);
		name_box->move(2, 2);
		name_box->resize(56, 23);
		name_box->setText(name);
		name_box->setFrame(FALSE);
		name_box->setReadOnly(TRUE);
		name_box->setAlignment(Qt::AlignCenter);
	}

	send1 = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	send1->move(3, 27);
	send1->resize(27, 50);
	send1->setValue(127);
	connect(send1, SIGNAL(valueChanged(int)), this, SLOT(slotSend1(int)));

	send2 = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	send2->move(30, 27);
	send2->resize(27, 50);
	send2->setValue(127);
	connect(send2, SIGNAL(valueChanged(int)), this, SLOT(slotSend2(int)));

	record = new QPushButton(this);
	record->move(3, 80);
	record->resize(18, 25);
	record->setText("R");
	record->setToggleButton(TRUE);
	record->setBackgroundColor(QColor(0xff,0,0));
	mixer->record_button_group->insert(record);
	connect(record, SIGNAL(pressed()), this, SLOT(slotRecord()));

	output = new QPushButton(this);
	output->move(21, 80);
	output->resize(18, 25);
	output->setText("O");
	output->setToggleButton(TRUE);
	output->setBackgroundColor(QColor(0,0xff,0));
	mixer->output_button_group->insert(output);
    connect(output, SIGNAL(pressed()), this, SLOT(slotOutput()));

	mute = new QPushButton(this);
	mute->move(39, 80);
	mute->resize(18, 25);
	mute->setText("M");
	mute->setToggleButton(TRUE);
	mute->setBackgroundColor(QColor(0,0,0xff));
    connect(mute, SIGNAL(pressed()), this, SLOT(slotMute()));

	fader = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	fader->move(5, 107);
	fader->resize(50, 141);
	fader->setValue(27);
	connect(fader, SIGNAL(valueChanged(int)), this, SLOT(slotFader(int)));
}

void usbua100mix_channel:: slotFader(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401105 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405003, 127 - value);
}

void usbua100mix_channel:: slotSend1(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401100 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00404007, 127 - value);
}

void usbua100mix_channel:: slotSend2(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401102 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00404009, 127 - value);
}

void usbua100mix_channel::slotRecord()
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405000, channel);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405000, 9);
}

void usbua100mix_channel::slotOutput()
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405001, channel);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405001, 9);
}

void usbua100mix_channel::slotMute()
{
    if (channel < 4)
		if (mute->isOn())
			((usbua100mix*)parent())->send_ua100_sysex_message(0x00401106 + ((channel + 1) << 8), 0);
		else
			((usbua100mix*)parent())->send_ua100_sysex_message(0x00401106 + ((channel + 1) << 8), 1);
}

void usbua100mix_channel::slotModeChange(int mode)
{
	((usbua100mix*)parent())->send_ua100_sysex_message(0x00401000, mode);
}


/***************************** Main Mixer Widget *************************************/

usbua100mix::usbua100mix(const char *file_name)
{
	resize(600, 300);
	setMinimumSize(300, 300);
	is_changed = 0;

	fileMenu = new QPopupMenu();
	fileMenu->insertItem("&Open...", this, SLOT(slotFileOpen()), CTRL+Key_O, USBUA100MIX_FILE_OPEN);
	fileMenu->insertItem("&Save", this, SLOT(slotFileSave()), CTRL+Key_S, USBUA100MIX_FILE_SAVE);
	fileMenu->insertItem("Save &as...", this, SLOT(slotFileSaveAs()), 0, USBUA100MIX_FILE_SAVE_AS);
	fileMenu->insertItem("M&IDI Port...", this, SLOT(slotFileMIDIPort()), 0, USBUA100MIX_FILE_MIDI_PORT);
	fileMenu->insertItem("E&xit", this, SLOT(slotFileExit()), CTRL+Key_Q, USBUA100MIX_FILE_EXIT);
	menuBar()->insertItem("&File", fileMenu);

	helpMenu=new QPopupMenu();
	helpMenu->insertItem("About...", this, SLOT(slotHelpAbout()), 0, USBUA100MIX_HELP_ABOUT);
	menuBar()->insertItem("&Help", helpMenu);

	statusBar()->message("Ready");

	effect_button_group = new QButtonGroup(this);
	effect_button_group->setExclusive(TRUE);

	full_effect_button = new QPushButton(this);
	full_effect_button->move(302, 25);
	full_effect_button->resize(48, 23);
	full_effect_button->setText("Full");
	full_effect_button->setToggleButton(TRUE);
	full_effect_button->setOn(TRUE);
	effect_button_group->insert(full_effect_button);
	connect(full_effect_button, SIGNAL(pressed()), this, SLOT(slotFullEffect()));

	insertion_effect_button = new QPushButton(this);
	insertion_effect_button->move(352, 25);
	insertion_effect_button->resize(48, 23);
	insertion_effect_button->setText("Ins.");
	insertion_effect_button->setToggleButton(TRUE);
	effect_button_group->insert(insertion_effect_button);
	connect(insertion_effect_button, SIGNAL(pressed()), this, SLOT(slotInsertionEffect()));

	system_effect_1_button = new QPushButton(this);
	system_effect_1_button->move(402, 25);
	system_effect_1_button->resize(48, 23);
	system_effect_1_button->setText("Sys1");
	system_effect_1_button->setToggleButton(TRUE);
	effect_button_group->insert(system_effect_1_button);
	connect(system_effect_1_button, SIGNAL(pressed()), this, SLOT(slotSystemEffect1()));

	system_effect_2_button = new QPushButton(this);
	system_effect_2_button->move(452, 25);
	system_effect_2_button->resize(48, 23);
	system_effect_2_button->setText("Sys2");
	system_effect_2_button->setToggleButton(TRUE);
	effect_button_group->insert(system_effect_2_button);
	connect(system_effect_2_button, SIGNAL(pressed()), this, SLOT(slotSystemEffect2()));

	record_button_group = new QButtonGroup(this);
	record_button_group->setExclusive(TRUE);
	output_button_group = new QButtonGroup(this);
	output_button_group->setExclusive(TRUE);

	insertion_effect = new usbua100mix_effect(this);
	insertion_effect->name_box->insertItem("None");
	insertion_effect->name_box->insertItem("00: Noise Supressor");
	insertion_effect->name_box->insertItem("01: Stereo Equalizer");
	insertion_effect->name_box->insertItem("02: Spectrum");
	insertion_effect->name_box->insertItem("03: Enhancer");
	insertion_effect->name_box->insertItem("04: Humanizer");
	insertion_effect->name_box->insertItem("05: Overdrive");
	insertion_effect->name_box->insertItem("06: Distortion");
	insertion_effect->name_box->insertItem("07: Phaser");
	insertion_effect->name_box->insertItem("08: Auto Wah");
	insertion_effect->name_box->insertItem("09: Rotary");
	insertion_effect->name_box->insertItem("10: Stereo Flanger");
	insertion_effect->name_box->insertItem("11: Step Flanger");
	insertion_effect->name_box->insertItem("12: Tremelo");
	insertion_effect->name_box->insertItem("13: Auto Pan");
	insertion_effect->name_box->insertItem("14: Compressor");
	insertion_effect->name_box->insertItem("15: Limiter");
	insertion_effect->name_box->insertItem("16: Hexa Chorus");
	insertion_effect->name_box->insertItem("17: Tremelo Chorus");
	insertion_effect->name_box->insertItem("18: Stereo Chorus");
	insertion_effect->name_box->insertItem("19: Space D Chorus");
	insertion_effect->name_box->insertItem("20: 3D Chorus");
	insertion_effect->name_box->insertItem("21: Stereo Delay");
	insertion_effect->name_box->insertItem("22: Modulation Delay");
	insertion_effect->name_box->insertItem("23: 3 Tap Delay");
	insertion_effect->name_box->insertItem("24: 4 Tap Delay");
	insertion_effect->name_box->insertItem("25: Time Control Delay");
	insertion_effect->name_box->insertItem("26: Reverb");
	insertion_effect->name_box->insertItem("27: Gate Reverb");
	insertion_effect->name_box->insertItem("28: 3D Delay");
	insertion_effect->name_box->insertItem("29: 2-voice Pitch Shifter");
	insertion_effect->name_box->insertItem("30: Feedback Pitch Shifter");
	insertion_effect->name_box->insertItem("31: 3D Auto");
	insertion_effect->name_box->insertItem("32: 3D Manual");
	insertion_effect->name_box->insertItem("33: Lo-Fi 1");
	insertion_effect->name_box->insertItem("34: Lo-Fi 2");
	insertion_effect->name_box->insertItem("35: Overdrive/Chorus");
	insertion_effect->name_box->insertItem("36: Overdrive/Flanger");
	insertion_effect->name_box->insertItem("37: Overdrive/Delay");
	insertion_effect->name_box->insertItem("38: Distortion/Chorus");
	insertion_effect->name_box->insertItem("39: Distortion/Flanger");
	insertion_effect->name_box->insertItem("40: Distortion/Delay");
	insertion_effect->name_box->insertItem("41: Enhancer/Chorus");
	insertion_effect->name_box->insertItem("42: Enhancer/Flanger");
	insertion_effect->name_box->insertItem("43: Enhancer/Delay");
	insertion_effect->name_box->insertItem("44: Chorus/Delay");
	insertion_effect->name_box->insertItem("45: Flanger/Delay");
	insertion_effect->name_box->insertItem("46: Chorus/Flanger");
	insertion_effect->name_box->insertItem("47: Rotary Multi");
	insertion_effect->name_box->insertItem("48: Guitar Multi1");
	insertion_effect->name_box->insertItem("49: Guitar Multi2");
	insertion_effect->name_box->insertItem("50: Guitar Multi3");
	insertion_effect->name_box->insertItem("51: Clean Guitar Multi1");
	insertion_effect->name_box->insertItem("52: Clean Guitar Multi2");
	insertion_effect->name_box->insertItem("53: Bass Multi");
	insertion_effect->name_box->insertItem("54: E.Piano Multi");
	insertion_effect->name_box->insertItem("55: Keyboard Multi");
	insertion_effect->name_box->insertItem("56: Chorus/Delay");
	insertion_effect->name_box->insertItem("57: Flanger/Delay");
	insertion_effect->name_box->insertItem("58: Chorus/Flanger");
	insertion_effect->name_box->insertItem("59: Overdrive/Distortion1,2");
	insertion_effect->name_box->insertItem("60: Overdrive/Distortion/Rotary");
	insertion_effect->name_box->insertItem("61: Overdrive/Distortion/Phaser");
	insertion_effect->name_box->insertItem("62: Overdrive/Distortion/Auto-wah");
	insertion_effect->name_box->insertItem("63: Phaser/Rotary");
	insertion_effect->name_box->insertItem("64: Phaser/Auto-wah");
	connect(insertion_effect->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeInsertionEffect(int)));

	system_effect_1 = new usbua100mix_effect(this, 4);
	system_effect_1->name_box->insertItem("None");
	system_effect_1->name_box->insertItem("Delay");
	system_effect_1->name_box->insertItem("Chorus");
	connect(system_effect_1->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeSystemEffect1(int)));

	system_effect_2 = new usbua100mix_effect(this, 5);
	system_effect_2->name_box->insertItem("None");
	system_effect_2->name_box->insertItem("Delay");
	system_effect_2->name_box->insertItem("Reverb");
	connect(system_effect_2->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeSystemEffect2(int)));

	full_effect = new usbua100mix_effect(this);
	full_effect->name_box->insertItem("None");
	full_effect->name_box->insertItem("01: High Quality Reverb");
	full_effect->name_box->insertItem("02: Microphone Simulator");
	full_effect->name_box->insertItem("03: Vocoder");
	full_effect->name_box->insertItem("04: Vocal Multi");
	full_effect->name_box->insertItem("05: Game with 3D Reverb");
	connect(full_effect->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeFullEffect(int)));
	full_effect_button->setOn(TRUE);

	channels[0] = new usbua100mix_channel(this, 0, "Mic1");
	channels[1] = new usbua100mix_channel(this, 1, "Mic2");
	channels[2] = new usbua100mix_channel(this, 2, "Wav1");
	channels[3] = new usbua100mix_channel(this, 3, "Wav2");
	channels[4] = new usbua100mix_channel(this, 4, "Main");

	if (file_name && file_name[0])
	{
		setCaption(file_name);
		load_mixer_settings(file_name);
	}

	setIcon(QPixmap(usbua100mix_xpm));
	setDockMenuEnabled(FALSE);

	strcpy(midi_port_name, "/dev/midi02");
	midi_port = open(midi_port_name, O_WRONLY);
	if (midi_port < 0)
		QMessageBox::information(this, strerror(errno), "Failure opening UA-100 Control Port", QMessageBox::Ok);
}

bool usbua100mix::ok_to_exit()
{
	if (!is_changed)
		return 1;

	int status = QMessageBox::information(this, "Save?", "Do you want to save settings?", QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);

	if (status == QMessageBox::No)
		return 1;

	else if (status == QMessageBox::Cancel)
		return 0;

	else
		slotFileSave();

	return 1;
}

int usbua100mix::send_ua100_sysex_message(unsigned int address, unsigned int value, int data_bytes = 1)
{
	if (midi_port < 0)
		return 0;

	int message_length = 13;
	unsigned char message[16] = { 0xF0, 0x41, 0x10, 0x00, 0x11, 0x12,   0x00, 0x00, 0x00, 0x00,   0x00,  0x00, 0x00, 0x00 };

	message[6] = (address >> 24) & 0xff;
	message[7] = (address >> 16) & 0xff;
	message[8] = (address >> 8) & 0xff;
	message[9] = address & 0xff;

	if (data_bytes == 1)
	{
		message[10] = value;
		message[11] = 128 - ((message[6] + message[7] + message[8] + message[9] + message[10]) & 0x7f);
		message[12] = 0xf7;
	}
	else
	{
		message[10] = (value >> 8) & 0xff;
		message[11] = value & 0xff;
		message[12] = 128 - ((message[6] + message[7] + message[8] + message[9] + message[10] + message[11]) & 0x7f);
		message[13] = 0xf7;
		message_length = 14;
	}

	char text[64] = "";
	sprintf(text, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
		message[0], message[1], message[2], message[3], message[4], message[5], message[6],
		message[7], message[8], message[9], message[10], message[11], message[12], message[13]);

	statusBar()->message(text);

	++is_changed;

	return write(midi_port, message, message_length);
}

int usbua100mix::save_mixer_settings(const char *file_name)
{
	FILE *file = fopen(file_name, "w");

	if (!file)
	{
		perror(file_name);
		QMessageBox::information(this, file_name, "Failure saving settings", QMessageBox::Ok);
		return 0;
	}

	fprintf(file, "[MODE]\nUA100CONTROLLER=True\n");
	if (full_effect)
		fprintf(file, "EFFMODE=2\n");
	else
		fprintf(file, "EFFMODE=1\n"); // ???
	fprintf(file, "SUBMODE=0\n\n");

	fprintf(file, "[INPUT]\nMICLINE=%d\n", channels[0]->input_mode->currentItem());
	fprintf(file, "0_PAN=64\n1_PAN=64\n");

	for (int channel = 0; channel < 4; ++channel)
    {
		fprintf(file, "%d_SEND1=%d\n", channel, 127 - channels[channel]->send1->value());
		fprintf(file, "%d_SEND2=%d\n", channel, 127 - channels[channel]->send2->value());
		fprintf(file, "%d_SUB=100\n", channel);
	}

	for (int channel = 0; channel < 4; ++channel)
		fprintf(file, "%d_FADER=%d\n", channel, 127 - channels[channel]->fader->value());

	fprintf(file, "\n[SYSTEM]\n");
	fprintf(file, "0_RETURN=%d\n0_SUB=100\n", 127 - channels[4]->send1->value());
	fprintf(file, "1_RETURN=%d\n1_SUB=100\n", 127 - channels[4]->send2->value());

	fprintf(file, "\n[OUTPUT]\n");

	if (channels[0]->record->isOn()) // ????
		fprintf(file, "1_SELECT=0\n");
	else if (channels[1]->record->isOn())
		fprintf(file, "1_SELECT=1\n");
	else if (channels[2]->record->isOn())
		fprintf(file, "1_SELECT=2\n");
	else if (channels[3]->record->isOn())
		fprintf(file, "1_SELECT=3\n");
	else
		fprintf(file, "1_SELECT=8\n");

	if (channels[0]->output->isOn()) // ????
		fprintf(file, "2_SELECT=0\n");
	else if (channels[1]->output->isOn())
		fprintf(file, "2_SELECT=1\n");
	else if (channels[2]->output->isOn())
		fprintf(file, "2_SELECT=2\n");
	else if (channels[3]->output->isOn())
		fprintf(file, "2_SELECT=3\n");
	else
		fprintf(file, "2_SELECT=8\n");

	fprintf(file, "0_FADER=%d\n", 127 - channels[4]->fader->value()); //?????
	fprintf(file, "1_FADER=100\n");

	fprintf(file, "\n[EFFECT]\n");
	for (int path = 0; path < 6; ++path)
		if (full_effect && (full_effect->path == path))
			fprintf(file, "%d_SWITCH=1\n", path);
		else if (insertion_effect && (full_effect->path == path))
			fprintf(file, "%d_SWITCH=1\n", path);
		else
			fprintf(file, "%d_SWITCH=0\n", path);

	if (full_effect->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT%02d]\n", full_effect->path + 1);
		fprintf(file, "TYPE=%d\n", full_effect->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", full_effect->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (full_effect->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, full_effect->parameter[scan]->get_value());
	}

	if (insertion_effect->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT%02d]\n", insertion_effect->path + 1);
		fprintf(file, "TYPE=%d\n", insertion_effect->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", insertion_effect->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (insertion_effect->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, insertion_effect->parameter[scan]->get_value());
	}

	if (system_effect_1->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT05]\n");
		fprintf(file, "TYPE=%d\n", system_effect_1->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", system_effect_1->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (system_effect_1->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, system_effect_1->parameter[scan]->get_value());
	}

	if (system_effect_2->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT06]\n");
		fprintf(file, "TYPE=%d\n", system_effect_2->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", system_effect_2->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (system_effect_2->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, system_effect_2->parameter[scan]->get_value());
	}

	fclose(file);

	is_changed = 0;

    return 0;
}

int usbua100mix::load_mixer_settings(const char *file_name)
{
	FILE *file = fopen(file_name, "r");
	char buffer[256];
	int effect_mode = 1; // 1=compact, 2=full
	int effect_path = 0; // 0=mic/line, 1=mic2, 2=wave1, 3=wave2, 4=sys1, 5=sys2
	int is_input = 1;
	usbua100mix_effect *current_effect = NULL;

	if (!file)
	{
		perror(file_name);
		QMessageBox::information(this, file_name, "Failure loading settings", QMessageBox::Ok);
		return 0;
	}

	channels[0]->input_mode->setCurrentItem(0);

	while (fgets(buffer, sizeof(buffer), file))
		if (strncmp(buffer, "EFFMODE=", 8) == 0)
			effect_mode = atoi(&buffer[8]);

		else if (strncmp(buffer, "MICLINE=", 8) == 0)
			channels[0]->input_mode->setCurrentItem(atoi(&buffer[8]));

		else if (strncmp(&buffer[2], "SEND1=", 6) == 0)
			channels[atoi(buffer)]->send1->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(&buffer[2], "SEND2=", 6) == 0)
			channels[atoi(buffer)]->send2->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "[OUTPUT]", 8) == 0)
			is_input = 0;

		else if (is_input && (strncmp(&buffer[2], "FADER=", 6) == 0))
			channels[atoi(buffer)]->fader->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "1_FADER=", 8) == 0)
			channels[4]->fader->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "0_RETURN=", 9) == 0)
			channels[4]->send1->setValue(127 - atoi(&buffer[9]));

		else if (strncmp(buffer, "1_RETURN=", 9) == 0)
			channels[4]->send2->setValue(127 - atoi(&buffer[9]));

		else if (strncmp(buffer, "1_SELECT=", 9) == 0)
		{
			if (channels[0]->input_mode->currentItem() == 0) // 2 microphones
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->output->setOn(1); break;
					case 1: channels[1]->output->setOn(1); break;
					case 2: channels[2]->output->setOn(1); break;
					case 3: channels[3]->output->setOn(1); break;
					default: channels[4]->output->setOn(1);
				}
			else // line
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->output->setOn(1); break;
					case 1: channels[2]->output->setOn(1); break;
					case 2: channels[3]->output->setOn(1); break;
					default: channels[4]->output->setOn(1);
				}
		}

		else if (strncmp(buffer, "2_SELECT=", 9) == 0)
		{
			if (channels[0]->input_mode->currentItem() == 0) // 2 microphones
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->record->setOn(1); break;
					case 1: channels[1]->record->setOn(1); break;
					case 2: channels[2]->record->setOn(1); break;
					case 3: channels[3]->record->setOn(1); break;
					default: channels[4]->record->setOn(1);
				}
			else // line
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->record->setOn(1); break;
					case 1: channels[2]->record->setOn(1); break;
					case 2: channels[3]->record->setOn(1); break;
					default: channels[4]->record->setOn(1);
				}
		}

		else if (strncmp(buffer, "[EFFECT", 7) == 0)
		{
			effect_path = atoi(&buffer[7]);
			if (effect_mode == 2)
			{
				current_effect = full_effect;
				full_effect->path_box->setCurrentItem(effect_path - 1);
			}
			else if (effect_path < 4)
			{
				current_effect = insertion_effect;
				insertion_effect->path_box->setCurrentItem(effect_path - 1);
			}
			else if (effect_path == 4)
				current_effect = system_effect_1;
			else if (effect_path == 5)
				current_effect = system_effect_2;
		}

		else if (strncmp(buffer, "TYPE=", 5) == 0)
		{
			if (current_effect == full_effect)
				slotChangeFullEffect(atoi(&buffer[5]) + 1);
			else if (current_effect == system_effect_1)
				slotChangeSystemEffect1(atoi(&buffer[5]) + 1);
			else if (current_effect == system_effect_2)
				slotChangeSystemEffect2(atoi(&buffer[5]) + 1);
		}

		else if (strncmp(buffer, "TYPE1=", 6) == 0)
		{
			int type1 = atoi(&buffer[6]);
			fgets(buffer, sizeof(buffer), file);
			int type2 = atoi(&buffer[6]);
			current_effect = insertion_effect;

			if (type1 == 0) // filter
				slotChangeInsertionEffect(1 + type2);
			else if (type1 == 1) // distortion
				slotChangeInsertionEffect(6 + type2);
			else if (type1 == 2) // modulation
				slotChangeInsertionEffect(8 + type2);
			else if (type1 == 3) // compressor
				slotChangeInsertionEffect(15 + type2);
			else if (type1 == 4) // chorus
				slotChangeInsertionEffect(17 + type2);
			else if (type1 == 5) // reverb
				slotChangeInsertionEffect(22 + type2);
			else if (type1 == 6) // pitch shift
				slotChangeInsertionEffect(30 + type2);
			else if (type1 == 7) // 2 in series
				slotChangeInsertionEffect(32 + type2);
			else if (type1 == 8) // 2 in series
				slotChangeInsertionEffect(36 + type2);
			else if (type1 == 9) // 3 in series
				slotChangeInsertionEffect(48 + type2);
			else if (type1 == 10) // 2 in parallel
				slotChangeInsertionEffect(57 + type2);
		}

		else if (strncmp(buffer, "VALUE_", 6) == 0)
		{
			int parameter_number = atoi(&buffer[6]);
			if (current_effect && current_effect->parameter[parameter_number])
				current_effect->parameter[parameter_number]->set_value(atoi(&buffer[9]));
		}

	is_changed = 0;

	return 0;
}

/********************** Slots ************************/

void usbua100mix::slotFileOpen()
{
	struct stat file_status;

	QString file_name = QFileDialog::getOpenFileName(0, "All Files (*.*);;Mixer Settings (*.ua1)", this);

	if (file_name.isEmpty())
		return;

	else if (stat(file_name, &file_status) < 0)
		QMessageBox::information(this, strerror(errno), "Failure opening file", QMessageBox::Ok);

	else
	{
		load_mixer_settings(file_name);
		setCaption(file_name);
		statusBar()->message("New Settings");
	}
}

void usbua100mix::slotFileSave()
{
	if (caption().isEmpty())
		slotFileSaveAs();

	else
		save_mixer_settings(caption());

	statusBar()->message("Mixer settings saved");
}

void usbua100mix::slotFileSaveAs()
{
	statusBar()->message("Saving file under new filename...");

	QString file_name = QFileDialog::getSaveFileName(NULL, "Mixer settings (*.uax);;All Files (*.*)", this);

	if (!file_name.isEmpty())
	{
		if (!strstr(file_name, ".uax"))
			file_name.append(".uax");

		setCaption(file_name);
		slotFileSave();
	}
	else
		statusBar()->message("Mixer settings not saved");
}

void usbua100mix::slotFileMIDIPort()
{
	strcpy(midi_port_name, QFileDialog::getOpenFileName("/dev/midi02", "All Files (*.*)", this));

	if (midi_port_name[0])
	{
		if (midi_port >= 0)
			close(midi_port);

		if ((midi_port = open(midi_port_name, O_WRONLY)) < 0)
			QMessageBox::information(this, strerror(errno), "Failure opening UA-100 Control Port", QMessageBox::Ok);
	}
}

void usbua100mix::slotFileExit()
{
	statusBar()->message("Exiting application...");
	if (ok_to_exit())
	      qApp->quit();

	statusBar()->message("");
}

void usbua100mix::slotHelpAbout()
{
	QMessageBox::about(this,"About...", USBUA100MIX_ABOUT);
}

void usbua100mix::slotFullEffect()
{
	full_effect->show();
	insertion_effect->hide();
	system_effect_1->hide();
	system_effect_2->hide();
}

void usbua100mix::slotInsertionEffect()
{
	full_effect->hide();
	insertion_effect->show();
	system_effect_1->hide();
	system_effect_2->hide();
}

void usbua100mix::slotSystemEffect1()
{
	full_effect->hide();
	insertion_effect->hide();
	system_effect_1->show();
	system_effect_2->hide();
}

void usbua100mix::slotSystemEffect2()
{
	full_effect->hide();
	insertion_effect->hide();
	system_effect_1->hide();
	system_effect_2->show();
}

void usbua100mix::slotChangeFullEffect(int index)
{
	send_ua100_sysex_message(0x00404000, 04); // full effects mode

	full_effect->clear_effect();
	insertion_effect->clear_effect();
	system_effect_1->clear_effect();
	system_effect_2->clear_effect();

	full_effect->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
	{
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_HIGH_QUALITY_REVERB;
		full_effect->parameter[1] = new usbua100mix_effect_parameter(full_effect, 2, 27, 32, 78, 1, "Pre", 0, 127, 0x70);
		full_effect->parameter[2] = new usbua100mix_effect_parameter(full_effect, 34, 27, 64, 78, 2, "Time", 0, 127, 120);
		full_effect->parameter[3] = new usbua100mix_effect_parameter(full_effect, 66, 27, 96, 78, 3, "Damp", 0, 127, 0x68);
		full_effect->parameter[4] = new usbua100mix_effect_parameter(full_effect, 98, 27, 128, 78, 4, "ER Delay", 0, 127, 8);
		full_effect->parameter[5] = new usbua100mix_effect_parameter(full_effect, 130, 27, 160, 78, 5, "ER Mix", 0, 127, 15);
		full_effect->parameter[6] = new usbua100mix_effect_parameter(full_effect, 162, 27, 192, 78, 6, "Diffusion", 0, 10, 9);

		full_effect->parameter[0] = new usbua100mix_effect_parameter(full_effect, 200, 27, 294, 50, 0, 3, "Room1", "Room2", "Stage1", "Stage2", "Hall1", "Hall2");
		full_effect->parameter[10] = new usbua100mix_effect_parameter(full_effect, 200, 52, 294, 75, 10, 1, "Lo Freq 200", "Lo Freq 400");
		full_effect->parameter[13] = new usbua100mix_effect_parameter(full_effect, 200, 77, 294, 100, 13, 0, "Mid1 Q 0.5", "Mid1 Q 1.0", "Mid1 Q 2.0", "Mid1 Q 4.0", "Mid1 Q 9.0");
		full_effect->parameter[16] = new usbua100mix_effect_parameter(full_effect, 200, 102, 294, 125, 16, 0, "Mid2 Q 0.5", "Mid2 Q 1.0", "Mid2 Q 2.0", "Mid2 Q 4.0", "Mid2 Q 9.0");
		full_effect->parameter[18] = new usbua100mix_effect_parameter(full_effect, 200, 127, 294, 150, 18, 1, "Hi Freq 4k", "Hi Freq 8k");

		full_effect->parameter[7] = new usbua100mix_effect_parameter(full_effect, 2, 80, 32, 131, 7, "Tone Lo", 0x34, 0x4c, 0x40);
		full_effect->parameter[8] = new usbua100mix_effect_parameter(full_effect, 34, 80, 64, 131, 8, "Tone Hi", 0x34, 0x4c, 0x40);
		full_effect->parameter[9] = new usbua100mix_effect_parameter(full_effect, 66, 80, 96, 131, 9, "Balance", 0, 127, 0x40);
		full_effect->parameter[12] = new usbua100mix_effect_parameter(full_effect, 98, 80, 128, 131, 12, "Mid1 Freq", 0, 0x7f, 0x48);
		full_effect->parameter[15] = new usbua100mix_effect_parameter(full_effect, 130, 80, 160, 131, 15, "Mid2 Freq", 0, 0x7f, 0x38);

		full_effect->parameter[11] = new usbua100mix_effect_parameter(full_effect, 2, 133, 32, 184, 11, "Lo Gain", 0x34, 0x4c, 0x45);
		full_effect->parameter[14] = new usbua100mix_effect_parameter(full_effect, 34, 133, 64, 184, 14, "Mid1 Gain", 0x34, 0x4c, 0x48);
		full_effect->parameter[17] = new usbua100mix_effect_parameter(full_effect, 66, 133, 96, 184, 17, "Mid2 Gain", 0x34, 0x4c, 0x38);
		full_effect->parameter[19] = new usbua100mix_effect_parameter(full_effect, 98, 133, 128, 184, 19, "Hi Gain", 0x34, 0x4c, 0x40);
	}

	else if (index == 2)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_MICROPHONE_SIMULATOR;

	else if (index == 3)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_VOCODER;

	else if (index == 4)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_VOCAL_MULTI;

	else if (index == 5)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_GAME;

	send_ua100_sysex_message(0x00400100 + (full_effect->path << 8), full_effect->effect_number, 2);
	send_ua100_sysex_message(0x00404001 + full_effect->path, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeInsertionEffect(int index)
{
	send_ua100_sysex_message(0x00404000, 03); // compact effects mode

	full_effect->clear_effect();
	insertion_effect->clear_effect();
	insertion_effect->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_NOISE_SUPPRESSOR;
		insertion_effect->parameter[0] = new usbua100mix_effect_parameter(insertion_effect, 2, 25, 22, 78, 0, "Threshold", 0, 0x7f, 10);
	}

	else if (index == 2)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_EQ;

		insertion_effect->parameter[1] = new usbua100mix_effect_parameter(insertion_effect, 2, 27, 32, 78, 1, "Lo Gain", 0x34, 0x4c, 0x45);
		insertion_effect->parameter[6] = new usbua100mix_effect_parameter(insertion_effect, 34, 27, 64, 78, 6, "Mid1 Gain", 0x34, 0x4c, 0x48);
		insertion_effect->parameter[9] = new usbua100mix_effect_parameter(insertion_effect, 66, 27, 96, 78, 9, "Mid2 Gain", 0x34, 0x4c, 0x38);
		insertion_effect->parameter[3] = new usbua100mix_effect_parameter(insertion_effect, 98, 27, 128, 78, 3, "Hi Gain", 0x34, 0x4c, 0x40);

		insertion_effect->parameter[0] = new usbua100mix_effect_parameter(insertion_effect, 130, 27, 294, 50, 0, 1, "Lo Freq 200", "Lo Freq 400");
		insertion_effect->parameter[5] = new usbua100mix_effect_parameter(insertion_effect, 130, 52, 294, 75, 5, 0, "Mid1 Q 0.5", "Mid1 Q 1.0", "Mid1 Q 2.0", "Mid1 Q 4.0", "Mid1 Q 9.0");
		insertion_effect->parameter[8] = new usbua100mix_effect_parameter(insertion_effect, 130, 77, 294, 100, 8, 0, "Mid2 Q 0.5", "Mid2 Q 1.0", "Mid2 Q 2.0", "Mid2 Q 4.0", "Mid2 Q 9.0");
		insertion_effect->parameter[2] = new usbua100mix_effect_parameter(insertion_effect, 130, 102, 294, 125, 2, 1, "Hi Freq 4k", "Hi Freq 8k");

		insertion_effect->parameter[10] = new usbua100mix_effect_parameter(insertion_effect, 2, 80, 32, 131, 19, "Level", 0, 127, 0x7f);
		insertion_effect->parameter[11] = new usbua100mix_effect_parameter(insertion_effect, 34, 80, 64, 131, 34, "Noise Suppressor", 0, 127, 10);
		insertion_effect->parameter[4] = new usbua100mix_effect_parameter(insertion_effect, 66, 80,96, 131, 4, "Mid1 Freq", 0, 0x7f, 0x48);
		insertion_effect->parameter[7] = new usbua100mix_effect_parameter(insertion_effect, 98, 80, 128, 131, 7, "Mid2 Freq", 0, 0x7f, 0x38);
	}

	else if (index == 3)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_SPECTRUM;
		insertion_effect->parameter[0] = new usbua100mix_effect_parameter(insertion_effect, 2, 27, 22, 78, 0, "250", 52, 76, 64);
		insertion_effect->parameter[1] = new usbua100mix_effect_parameter(insertion_effect, 24, 27, 44, 78, 1, "500", 52, 76, 64);
		insertion_effect->parameter[2] = new usbua100mix_effect_parameter(insertion_effect, 46, 27, 66, 78, 2, "1K", 52, 76, 64);
		insertion_effect->parameter[3] = new usbua100mix_effect_parameter(insertion_effect, 68, 27, 88, 78, 3, "1.3", 52, 76, 64);
		insertion_effect->parameter[4] = new usbua100mix_effect_parameter(insertion_effect, 90, 27, 110, 78, 4, "2K", 52, 76, 64);
		insertion_effect->parameter[5] = new usbua100mix_effect_parameter(insertion_effect, 112, 27, 132, 78, 5, "3.2", 52, 76, 64);
		insertion_effect->parameter[6] = new usbua100mix_effect_parameter(insertion_effect, 134, 27, 154, 78, 6, "4K", 52, 76, 64);
		insertion_effect->parameter[7] = new usbua100mix_effect_parameter(insertion_effect, 156, 27, 176, 78, 7, "8K", 52, 76, 64);
		insertion_effect->parameter[8] = new usbua100mix_effect_parameter(insertion_effect, 178, 27, 198, 78, 18, "Pan", 0, 127, 64);
		insertion_effect->parameter[9] = new usbua100mix_effect_parameter(insertion_effect, 200, 27, 220, 78, 19, "Out", 0, 127, 100);
		insertion_effect->parameter[10] = new usbua100mix_effect_parameter(insertion_effect, 222, 27, 242, 78, 34, "Gate", 0, 127, 10);
		insertion_effect->parameter[11] = new usbua100mix_effect_parameter(insertion_effect, 256, 27, 298, 78, 8, 0, "Q0.5", "Q1.0", "Q2.0", "Q4.0", "Q9.0");
	}

	else if (index == 4)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER;
		insertion_effect->parameter[0] = new usbua100mix_effect_parameter(insertion_effect, 2, 27, 32, 78, 0, "Sensitivity", 0, 127, 64);
		insertion_effect->parameter[1] = new usbua100mix_effect_parameter(insertion_effect, 34, 27, 64, 78, 1, "Mix", 0, 127, 127);
		insertion_effect->parameter[2] = new usbua100mix_effect_parameter(insertion_effect, 66, 27, 96, 78, 16, "Lo Gain", 0x34, 0x4c, 0x40);
		insertion_effect->parameter[3] = new usbua100mix_effect_parameter(insertion_effect, 98, 27, 128, 78, 17, "Hi Gain", 0x34, 0x4c, 0x40);
		insertion_effect->parameter[4] = new usbua100mix_effect_parameter(insertion_effect, 130, 27, 160, 78, 19, "Level", 0, 127, 127);
		insertion_effect->parameter[5] = new usbua100mix_effect_parameter(insertion_effect, 162, 27, 192, 78, 34, "Noise Suppressor", 0, 127, 10);
	}

	else if (index == 5)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_HUMANIZER;

	else if (index == 6)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE;

	else if (index == 7)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION;

	else if (index == 8)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER;

	else if (index == 9)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_AUTO_WAH;

	else if (index == 10)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ROTARY;

	else if (index == 11)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_FLANGER;

	else if (index == 12)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEP_FLANGER;

	else if (index == 13)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TREMELO;

	else if (index == 14)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_AUTO_PAN;

	else if (index == 15)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_COMPRESSOR;

	else if (index == 16)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LIMITER;

	else if (index == 17)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_HEXA_CHORUS;

	else if (index == 18)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TREMELO_CHORUS;

	else if (index == 19)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_CHORUS;

	else if (index == 20)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_SPACE_D_CHORUS;

	else if (index == 21)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_CHORUS;

	else if (index == 22)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_DELAY;

	else if (index == 23)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_MODULATION_DELAY;

	else if (index == 24)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3_TAP_DELAY;

	else if (index == 25)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_4_TAP_DELAY;

	else if (index == 26)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TIME_CONTROL_DELAY;

	else if (index == 27)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_REVERB;

	else if (index == 28)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GATE_REVERB;

	else if (index == 29)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_DELAY;

	else if (index == 30)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_2_VOICE_PITCH;

	else if (index == 31)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FEEDBACK_PITCH;

	else if (index == 32)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_AUTO;

	else if (index == 33)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_MANUAL;

	else if (index == 34)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LO_FI_1;

	else if (index == 35)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LO_FI_2;

	else if (index == 36)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_CHORUS;

	else if (index == 37)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_FLANGER;

	else if (index == 38)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DELAY;

	else if (index == 39)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_CHORUS;

	else if (index == 40)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_FLANGER;

	else if (index == 41)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_DELAY;

	else if (index == 42)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_CHORUS;

	else if (index == 43)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_FLANGER;

	else if (index == 44)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_DELAY;

	else if (index == 45)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY;

	else if (index == 46)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY;

	else if (index == 47)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER;

	else if (index == 48)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ROTARY_MULTI;

	else if (index == 49)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI1;

	else if (index == 50)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI2;

	else if (index == 51)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI3;

	else if (index == 52)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI1;

	else if (index == 53)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI2;

	else if (index == 54)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_BASS_BULTI;

	else if (index == 55)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PIANO_MULTI;

	else if (index == 56)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_KEYBOARD_MULTI;

	else if (index == 57)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY_PARALLEL;

	else if (index == 58)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY_PARALLEL;

	else if (index == 59)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER_PARALLEL;

	else if (index == 60)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DISTORTION;

	else if (index == 61)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_ROTARY;

	else if (index == 62)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_PHASER;

	else if (index == 63)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_AUTOWAH;

	else if (index == 64)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER_ROTARY;

	else if (index == 65)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER_AUTOWAH;

	send_ua100_sysex_message(0x00400100 + (insertion_effect->path << 8), insertion_effect->effect_number, 2);
	send_ua100_sysex_message(0x00404001 + insertion_effect->path, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeSystemEffect1(int index)
{
	send_ua100_sysex_message(0x00404000, 03); // compact effects mode

	full_effect->clear_effect();
	system_effect_1->clear_effect();
	system_effect_1->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
		system_effect_1->effect_number = USBUA100MIX_EFFECT_SYS1_DELAY;

	else if (index == 2)
		system_effect_1->effect_number = USBUA100MIX_EFFECT_SYS1_CHORUS;

	send_ua100_sysex_message(0x00400500, system_effect_1->effect_number, 2);
	send_ua100_sysex_message(0x00404005, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeSystemEffect2(int index)
{
	send_ua100_sysex_message(0x00404000, 03); // compact effects mode

	full_effect->clear_effect();
	system_effect_2->clear_effect();
	system_effect_2->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
		system_effect_2->effect_number = USBUA100MIX_EFFECT_SYS2_DELAY;

	else if (index == 2)
	{
		system_effect_2->effect_number = USBUA100MIX_EFFECT_SYS2_REVERB;
		system_effect_2->parameter[0] = new usbua100mix_effect_parameter(system_effect_2, 154, 2, 298, 25, 0, 3, "Room1", "Room2", "Stage1", "Stage2", "Hall1", "Hall2");
		system_effect_2->parameter[1] = new usbua100mix_effect_parameter(system_effect_2, 2, 25, 32, 78, 1, "Pre", 0, 127, 0x70);
		system_effect_2->parameter[2] = new usbua100mix_effect_parameter(system_effect_2, 34, 25, 64, 78, 2, "Time", 0, 127, 120);
		system_effect_2->parameter[3] = new usbua100mix_effect_parameter(system_effect_2, 66, 25, 96, 78, 3, "Damp", 0, 127, 0x67);
		system_effect_2->parameter[4] = new usbua100mix_effect_parameter(system_effect_2, 98, 25, 128, 78, 4, "Lo", 0x34, 0x4c, 0x40);
		system_effect_2->parameter[5] = new usbua100mix_effect_parameter(system_effect_2, 130, 25, 160, 78, 5, "Hi", 0x34, 0x4c, 0x40);
		system_effect_2->parameter[6] = new usbua100mix_effect_parameter(system_effect_2, 162, 25, 192, 78, 6, "Balance", 0, 127, 0x40);
	}

	send_ua100_sysex_message(0x00400600, system_effect_2->effect_number, 2);
	send_ua100_sysex_message(0x00404006, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}


/********************** Events ************************/

void usbua100mix::closeEvent(QCloseEvent *event)
{
	if(ok_to_exit())
		QWidget::closeEvent(event);
}

void usbua100mix::keyPressEvent(QKeyEvent *event)
{
	/* QMessageBox::information(this, "Keystrokes", "Feature Not Implemented", QMessageBox::Ok); */

	QMainWindow::keyPressEvent(event);
}



/****************************** Effect Base Widget **************************************/

usbua100mix_effect::usbua100mix_effect(usbua100mix *mixer, int path) : QWidget(mixer)
{
	move(302, 50);
	resize(296, 225);

	name_box = new QComboBox(this);
	name_box->move(2, 2);
	name_box->resize(196, 23);

	if (path >= 0)
	{
		this->path = path;
		path_box = NULL;
	}
	else
	{
		path_box = new QComboBox(this);
		path_box->move(200, 2);
		path_box->resize(94, 23);
		path_box->insertItem("Line/Mic 1");
		path_box->insertItem("Mic 2");
		path_box->insertItem("Wave 1");
		path_box->insertItem("Wave 2");
		path_box->insertItem("Sys 1");
		path_box->insertItem("Sys 2");
		path_box->setCurrentItem(0);
		path_box->setEditable(FALSE);
		connect(path_box, SIGNAL(activated(int)), this, SLOT(slotPathChanged(int)));
		this->path = 0;
	}

	setBackgroundMode(QWidget::PaletteMid);

	this->effect_number = 0;
	this->mixer = mixer;
	memset(parameter, 0, sizeof(parameter));
}

void usbua100mix_effect::clear_effect()
{
	mixer->send_ua100_sysex_message(0x00404001 + path, 0);

	for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
		if (parameter[scan])
		{
			delete parameter[scan];
			parameter[scan] = NULL;
		}

	name_box->setCurrentItem(0);
}

void usbua100mix_effect::slotPathChanged(int new_path)
{
	path = new_path;
	mixer->send_ua100_sysex_message(0x00404001 + path, 1);
}

usbua100mix_effect_parameter::usbua100mix_effect_parameter(usbua100mix_effect *effect, int left_x, int top_y, int right_x, int bottom_y, int parameter_number, char *text, int low_value, int high_value, int initial_value)
{
	this->effect = effect;
	this->parameter_number = parameter_number;

	combo_box = NULL;
	slider = new QSlider (effect);
	slider->move(left_x, top_y);
	slider->resize(right_x - left_x, bottom_y - top_y);
	slider->setOrientation(QSlider::Vertical);
	slider->setMinValue(low_value);
	slider->setMaxValue(high_value);
	slider->setValue(127 - initial_value);
	slotSliderChanged(initial_value);
	QToolTip::add(slider, text);

	connect(slider, SIGNAL(valueChanged(int)), this, SLOT(slotSliderChanged(int)));
	slider->show();
}

usbua100mix_effect_parameter::usbua100mix_effect_parameter(usbua100mix_effect *effect, int left_x, int top_y, int right_x, int bottom_y, int parameter_number, int initial_value, char *text1, char *text2, char *text3, char *text4, char *text5, char *text6)
{
	this->effect = effect;
	this->parameter_number = parameter_number;

	slider = NULL;
	combo_box = new QComboBox(effect);
	combo_box->move(left_x, top_y);
	combo_box->resize(right_x - left_x, bottom_y - top_y);
	if (text1)
		combo_box->insertItem(text1);
	if (text2)
		combo_box->insertItem(text2);
	if (text3)
		combo_box->insertItem(text3);
	if (text4)
		combo_box->insertItem(text4);
	if (text5)
		combo_box->insertItem(text5);
	if (text6)
		combo_box->insertItem(text6);
	combo_box->setCurrentItem(initial_value);
	combo_box->setEditable(FALSE);
	slotComboBoxChanged(0);

	connect(combo_box, SIGNAL(activated(int)), this, SLOT(slotComboBoxChanged(int)));
	combo_box->show();
}

usbua100mix_effect_parameter::~usbua100mix_effect_parameter()
{
	if (slider)
		delete slider;

	if (combo_box)
		delete combo_box;

	slider = NULL;
	combo_box = NULL;
}

int usbua100mix_effect_parameter::get_value()
{
	if (this == NULL)
		return 0;
	else if (combo_box)
		return combo_box->currentItem();
	else if (slider)
		return 127 - slider->value();
	else
		return 0;
}

void usbua100mix_effect_parameter::set_value(int value)
{
	if (this == NULL)
		return;
	else if (combo_box)
		combo_box->setCurrentItem(value);
	else if (slider)
		slider->setValue(127 - value);
}

void usbua100mix_effect_parameter::slotSliderChanged(int value)
{
	if (effect && effect->mixer && (parameter_number >= 0))
		effect->mixer->send_ua100_sysex_message(0x00400103 + (effect->path << 8) + parameter_number, 127 - value);
}

void usbua100mix_effect_parameter::slotComboBoxChanged(int value)
{
	if (effect && effect->mixer && (parameter_number >= 0))
		effect->mixer->send_ua100_sysex_message(0x00400103 + (effect->path << 8) + parameter_number, value);

}

/********************** Main Program Entry Function ************************/

int main(int argc, char *argv[])
{
	usbua100mix_application application(argc, argv);
	application.connect(&application, SIGNAL(lastWindowClosed()), &application, SLOT(quit()));

	char *load_file_name = NULL;
	if (argc > 1)
		load_file_name = argv[1];

	application.mixer = new usbua100mix(load_file_name);

	application.mixer->show();

	return application.exec();
}
